@@ -2,6 +2,7 @@ from django.utils import unittest
 from django.core.validators import validate_email, validate_slug, URLValidator
 from django.utils.timezone import utc
 from django.core.files.images import ImageFile
+from django.core.management import call_command
 
 import string
 import datetime
@@ -9,9 +10,14 @@ import os
 import six
 
 from sampledatahelper.helper import SampleDataHelper
+from sampledatahelper.model_helper import ModelDataHelper
 from sampledatahelper.exceptions import ParameterError, NotChoicesFound
 from sampledatahelper.mixins import image_mixin
 
+from .models import TestModel, TestRelatedModel
+
+call_command('syncdb', interactive=False)
+
 
 class TestNumberHelpers(unittest.TestCase):
     @classmethod
@@ -253,15 +259,17 @@ class TestTimeHelpers(unittest.TestCase):
             )
 
     def test_future_datetime(self):
+        now = datetime.datetime.utcnow().replace(tzinfo=utc)
         value = self.sd.future_datetime()
         self.assertTrue(isinstance(value, datetime.datetime))
 
-        self.assertTrue(value >= datetime.datetime.utcnow().replace(tzinfo=utc))
-        self.assertTrue(value <= (datetime.datetime.utcnow().replace(tzinfo=utc) + datetime.timedelta(minutes=1440)))
+        self.assertTrue(value >= now)
+        self.assertTrue(value <= (now + datetime.timedelta(minutes=1440)))
 
+        now = datetime.datetime.utcnow().replace(tzinfo=utc)
         value = self.sd.future_datetime(1, 10)
-        self.assertTrue(value >= datetime.datetime.utcnow().replace(tzinfo=utc))
-        self.assertTrue(value <= (datetime.datetime.utcnow().replace(tzinfo=utc) + datetime.timedelta(minutes=10)))
+        self.assertTrue(value >= now)
+        self.assertTrue(value <= (now + datetime.timedelta(minutes=10)))
 
         with self.assertRaises(ParameterError):
             self.sd.future_datetime(100, 0)
@@ -314,6 +322,10 @@ class TestTimeHelpers(unittest.TestCase):
         with self.assertRaises(ParameterError):
             self.sd.datetime(100, 0)
 
+    def test_time(self):
+        value = self.sd.time()
+        self.assertTrue(isinstance(value, datetime.time))
+
 
 class TestLocalizedHelpers(unittest.TestCase):
     @classmethod
@@ -532,6 +544,7 @@ class TestImageHelpers(unittest.TestCase):
         with self.assertRaises(ParameterError):
             value = self.sd.image(100, 0)
 
+
 class TestOtherHelpers(unittest.TestCase):
     @classmethod
     def setUpClass(cls):
@@ -540,6 +553,51 @@ class TestOtherHelpers(unittest.TestCase):
     def test_boolean(self):
         self.assertTrue(isinstance(self.sd.boolean(), bool))
 
+    def test_ipv4(self):
+        value = self.sd.ipv4()
+        self.assertTrue(isinstance(value, six.string_types))
+        self.assertEqual(len(value.split('.')), 4)
+
+    def test_ipv6(self):
+        value = self.sd.ipv6()
+        self.assertTrue(isinstance(value, six.string_types))
+        self.assertEqual(len(value.split(':')), 8)
+
+    def test_mac_address(self):
+        value = self.sd.mac_address()
+        self.assertTrue(isinstance(value, six.string_types))
+        self.assertEqual(len(value.split(':')), 6)
+
+    def test_path(self):
+        value = self.sd.path()
+        self.assertTrue(isinstance(value, six.string_types))
+
+        value = self.sd.path(absolute=False, min_levels=5, max_levels=5)
+        self.assertEqual(len(value.split('/')), 5)
+
+        value = self.sd.path(absolute=True, min_levels=5, max_levels=5)
+        self.assertEqual(len(value.split('/')), 6)
+
+        value = self.sd.path(extension=".jpg")
+        self.assertEqual(value[-4:], ".jpg")
+
+        with self.assertRaises(ParameterError):
+            self.sd.path(absolute=True, min_levels=10, max_levels=5)
+
+    def test_hex_chars(self):
+        value = self.sd.hex_chars()
+        self.assertTrue(isinstance(value, six.string_types))
+        self.assertTrue(len(value) >= 1)
+        self.assertTrue(len(value) <= 5)
+
+        value = self.sd.hex_chars(5, 5)
+        self.assertTrue(len(value) == 5)
+
+        self.assertEqual(self.sd.hex_chars(0, 0), '')
+
+        with self.assertRaises(ParameterError):
+            value = self.sd.hex_chars(10, 5)
+
     def test_choice(self):
         self.assertEqual(self.sd.choice([10]), 10)
 
@@ -564,10 +622,117 @@ class TestOtherHelpers(unittest.TestCase):
         with self.assertRaises(ParameterError):
             self.sd.choices_key([10])
 
-    # TODO
-    #def test_db_object(self):
-    #    pass
+    def test_db_object(self):
+        mdh = ModelDataHelper()
 
-    # TODO
-    #def test_db_object_from_queryset(self):
-    #    pass
+        with self.assertRaises(NotChoicesFound):
+            self.sd.db_object(TestRelatedModel)
+
+        mdh.fill_model(TestRelatedModel, 1)
+
+        self.assertTrue(isinstance(
+            self.sd.db_object(TestRelatedModel),
+            TestRelatedModel
+        ))
+        TestRelatedModel.objects.all().delete()
+
+    def test_db_object_from_queryset(self):
+        mdh = ModelDataHelper()
+
+        with self.assertRaises(NotChoicesFound):
+            self.sd.db_object_from_queryset(TestRelatedModel.objects.all())
+
+        mdh.fill_model(TestRelatedModel, 10)
+
+        self.assertTrue(isinstance(
+            self.sd.db_object_from_queryset(TestRelatedModel.objects.all()),
+            TestRelatedModel
+        ))
+        TestRelatedModel.objects.all().delete()
+
+
+class TestModelDataHelpers(unittest.TestCase):
+    @classmethod
+    def setUpClass(cls):
+        cls.mdh = ModelDataHelper()
+        cls.mdh.fill_model(TestRelatedModel, 10)
+
+    @classmethod
+    def tearDownClass(cls):
+        TestRelatedModel.objects.all().delete()
+
+    def test_fill_model_instance(self):
+        instance = TestModel()
+        self.mdh.fill_model_instance(instance)
+
+        with self.assertRaises(ParameterError):
+            self.mdh.fill_model_instance(unittest.TestCase)
+
+        self.mdh.fill_model_instance(instance, integer=lambda _, sd: sd.int(5, 5))
+        self.assertEqual(instance.integer, 5)
+
+        self.mdh.fill_model_instance(instance, integer=15)
+        self.assertEqual(instance.integer, 15)
+
+    def test_fill_model(self):
+        self.mdh.fill_model(TestModel, 5)
+
+        with self.assertRaises(ParameterError):
+            self.mdh.fill_model(TestModel, 0)
+
+        with self.assertRaises(ParameterError):
+            self.mdh.fill_model(unittest.TestCase, 5)
+
+        TestModel.objects.all().delete()
+
+        self.mdh.fill_model(TestModel, 5, integer=lambda _, sd: sd.int(5, 5))
+        self.assertEqual(TestModel.objects.all()[0].integer, 5)
+
+        TestModel.objects.all().delete()
+
+        self.mdh.fill_model(TestModel, 5, integer=15)
+        self.assertEqual(TestModel.objects.all()[0].integer, 15)
+
+        TestModel.objects.all().delete()
+
+        self.mdh.fill_model(TestModel, 5, integer=lambda instance, _: instance.small_integer * 2)
+        test_instance = TestModel.objects.all()[0]
+        self.assertEqual(test_instance.integer, test_instance.small_integer * 2)
+
+        TestModel.objects.all().delete()
+
+        self.mdh.fill_model(TestModel, 5, ('integer', lambda _, sd: sd.int(5, 5)))
+        self.assertEqual(TestModel.objects.all()[0].integer, 5)
+
+        TestModel.objects.all().delete()
+
+        self.mdh.fill_model(TestModel, 5, ('integer', 15))
+        self.assertEqual(TestModel.objects.all()[0].integer, 15)
+
+        TestModel.objects.all().delete()
+
+        self.mdh.fill_model(TestModel, 5, ('integer', lambda instance, _: instance.small_integer * 2))
+        test_instance = TestModel.objects.all()[0]
+        self.assertEqual(test_instance.integer, test_instance.small_integer * 2)
+
+class TestSampleDataHelperCommand(unittest.TestCase):
+    @classmethod
+    def setUpClass(cls):
+        TestRelatedModel.objects.all().delete()
+        TestModel.objects.all().delete()
+
+    @classmethod
+    def tearDownClass(cls):
+        TestRelatedModel.objects.all().delete()
+        TestModel.objects.all().delete()
+
+    def test_calling_command(self):
+        call_command('sampledatafiller', interactive=False)
+
+        self.assertEqual(TestRelatedModel.objects.all().count(), 10)
+        self.assertEqual(TestModel.objects.all().count(), 5)
+
+        test_instance = TestModel.objects.all()[0]
+        self.assertEqual(test_instance.integer, test_instance.small_integer*2)
+        self.assertEqual(test_instance.positive_integer, 8)
+        self.assertTrue(test_instance.small_integer >= 1 and test_instance.small_integer <= 10)
